## Modeling Gas Flow Directions as State Variables:
# Does It Provide More Flexibility to Power Systems?
# Power Systems Computation Conference(PSCC) 2022, Electric Power Systems Research
# Code written by Junesoo Shin, s184386@student.dtu.dk
# 23.04.2022

function load_data(volatility,compressor_switch,percentage)
	PowerUnit_df = DataFrame(XLSX.readtable("data/input_data.xlsx", "PowerUnit")...)
	WindUnit_df = DataFrame(XLSX.readtable("data/input_data.xlsx", "WindUnit")...)
	WindFactor_df = DataFrame(XLSX.readtable("data/input_data.xlsx", "WindFactor")...)
	GasProducer_df = DataFrame(XLSX.readtable("data/input_data.xlsx", "GasProducer")...)
	ELNetwork_df = DataFrame(XLSX.readtable("data/input_data.xlsx", "ELNetwork")...)
	NGNetwork_df = DataFrame(XLSX.readtable("data/input_data.xlsx", "NGNetwork")...)
	NGPressure_df = DataFrame(XLSX.readtable("data/input_data.xlsx", "NGPressure")...)
	DemandData_df = DataFrame(XLSX.readtable("data/input_data.xlsx", "DemandData")...)
	ELShare_df = DataFrame(XLSX.readtable("data/input_data.xlsx", "ELShare")...)
	NGShare_df = DataFrame(XLSX.readtable("data/input_data.xlsx", "NGShare")...)
	W_df = CSV.read("data/windevents.csv", DataFrame)

	PP = Dict("i$I"=>Dict("Pmax"=>PowerUnit_df[I,names(PowerUnit_df)[3]],
	                      "ELnode"=>"n$(PowerUnit_df[I,names(PowerUnit_df)[4]])",
	                      "NGnode"=>"m$(PowerUnit_df[I,names(PowerUnit_df)[5]])",
	                      "type"=>PowerUnit_df[I,names(PowerUnit_df)[6]],
	                      "eta"=>PowerUnit_df[I,names(PowerUnit_df)[7]],
	                      "C_E"=>PowerUnit_df[I,names(PowerUnit_df)[8]])
	                      for I in 1:nrow(PowerUnit_df))

	#Injection ratio calculated to match (total W capacity/total EL demand = 20%,30%...)
	inj_level=Dict("20%"=>0.658, "30%"=>1, "40%"=>1.32, "50%"=>1.648, "60%"=>1.98,"70%"=>2.31,"80%"=>2.635)

	Windfarm = Dict("j$J"=>Dict("Wmax"=>WindUnit_df[J,names(WindUnit_df)[3]]*inj_level[percentage],
	                            "ELnode"=>"n$(WindUnit_df[J,names(WindUnit_df)[2]])")
	                            for J in 1:nrow(WindUnit_df))
	if volatility == "high"
		WindFactor = Dict("t$T"=>Dict("j1"=>WindFactor_df[T,names(WindFactor_df)[2]],
		                              "j2"=>WindFactor_df[T,names(WindFactor_df)[3]])
		                              for T in 1:nrow(WindFactor_df))
	elseif volatility == "low"
		#normalize values
		W_df_share = (W_df[1:240,3].-minimum(W_df[1:240,3]))./(maximum(W_df[1:240,3])-minimum(W_df[1:240,3]))
		#make mean of the following values same (so that only volatility is different)
		j1_mean = mean([WindFactor_df[T,names(WindFactor_df)[2]] for T in 1:nrow(WindFactor_df)])
		j2_mean = mean([WindFactor_df[T,names(WindFactor_df)[3]] for T in 1:nrow(WindFactor_df)])
		#1:24 and 193:216 were chosen as the volatile profiles
		new_j1 = W_df_share[1:24].-(mean(W_df_share[1:24])-j1_mean)
		new_j2 = W_df_share[193:216].-(mean(W_df_share[193:216])-j2_mean)
		WindFactor = Dict("t$i" => Dict("j1"=>new_j1[i],"j2"=>new_j2[i]) for i in 1:24)
	end
	GasSupply = Dict("k$K"=>Dict("Gmax"=>GasProducer_df[K,names(GasProducer_df)[3]],
	                             "C_G"=>GasProducer_df[K,names(GasProducer_df)[5]],
	                             "NGnode"=>"m$(GasProducer_df[K,names(GasProducer_df)[4]])")
	                             for K in 1:nrow(GasProducer_df))
	ELDemand = Dict("d$D"=>Dict("share"=>ELShare_df[D,names(ELShare_df)[2]],
	                            "node"=>"n$(ELShare_df[D,names(ELShare_df)[3]])")
	                            for D in 1:nrow(ELShare_df))
	NGDemand = Dict("d$D"=>Dict("share"=>NGShare_df[D,names(NGShare_df)[2]],
	                            "node"=>"m$(NGShare_df[D,names(NGShare_df)[3]])")
	                            for D in 1:nrow(NGShare_df))
	Demand = Dict("t$T"=>Dict("EL"=>DemandData_df[T,names(DemandData_df)[2]],
	                          "NG"=>DemandData_df[T,names(DemandData_df)[3]],
	                          "prev"=>"t$(T-1)")
	                          for T in 1:nrow(DemandData_df))
	Lines = Dict("l$L"=>Dict("From"=>"n$(ELNetwork_df[L,names(ELNetwork_df)[2]])",
	                         "To"=>"n$(ELNetwork_df[L,names(ELNetwork_df)[3]])",
	                         "B"=>1/ELNetwork_df[L,names(ELNetwork_df)[4]],
	                         "capacity"=>ELNetwork_df[L,names(ELNetwork_df)[5]])
	                         for L in 1:nrow(ELNetwork_df))
	Pipelines = Dict("z$Z"=>Dict("From"=>"m$(NGNetwork_df[Z,names(NGNetwork_df)[2]])",
	                             "To"=>"m$(NGNetwork_df[Z,names(NGNetwork_df)[3]])",
	                             "K_f"=>NGNetwork_df[Z,names(NGNetwork_df)[4]],
	                             "type"=>NGNetwork_df[Z,names(NGNetwork_df)[5]],
								 compressor_switch == true ?
								 "Gamma"=>NGNetwork_df[Z,names(NGNetwork_df)[6]] :
								 "Gamma"=>1,
	                             "K_h"=>NGNetwork_df[Z,names(NGNetwork_df)[7]],
	                             "H_ini"=>NGNetwork_df[Z,names(NGNetwork_df)[8]])
	                             for Z in 1:nrow(NGNetwork_df))

	NGPr = Dict("m$M"=>Dict("minPr"=>NGPressure_df[M,names(NGPressure_df)[2]],
	                        "maxPr"=>NGPressure_df[M,names(NGPressure_df)[3]])
	                        for M in 1:nrow(NGPressure_df))

	#Create fixed pressure point sets
	Random.seed!(1000)
	V_set = 50
	fixPR_set = Dict("v$V"=>Dict("m$M"=>rand(NGPr["m$M"]["minPr"]:NGPr["m$M"]["maxPr"])
	                        for M in 1:nrow(NGPressure_df)) for V in 1:V_set)
	# V_choice = [15,20,33,44] #Determined from choose V plots, selected from the set of 50
	V_choice = [2,4,15,20,44]
	fixPR = Dict("v$V" => fixPR_set["v$V"] for V in V_choice)

	#values are rounded to 1 digit!
	KI_plus = Dict("z$Z"=>Dict("v$V"=>fixPR["v$V"][Pipelines["z$Z"]["From"]]^2 > fixPR["v$V"][Pipelines["z$Z"]["To"]]^2
						    ? round(Pipelines["z$Z"]["K_f"]*fixPR["v$V"][Pipelines["z$Z"]["From"]]/sqrt(fixPR["v$V"][Pipelines["z$Z"]["From"]]^2-fixPR["v$V"][Pipelines["z$Z"]["To"]]^2),digits=1)
						    : 0 for V in V_choice) #1:length(keys(fixPR))
						    for Z in 1:nrow(NGNetwork_df))
	KO_plus = Dict("z$Z"=>Dict("v$V"=>fixPR["v$V"][Pipelines["z$Z"]["From"]]^2 > fixPR["v$V"][Pipelines["z$Z"]["To"]]^2
						    ? round(Pipelines["z$Z"]["K_f"]*fixPR["v$V"][Pipelines["z$Z"]["To"]]/sqrt(fixPR["v$V"][Pipelines["z$Z"]["From"]]^2-fixPR["v$V"][Pipelines["z$Z"]["To"]]^2),digits=1)
						    : 0 for V in V_choice)
						    for Z in 1:nrow(NGNetwork_df))
	KI_minus = Dict("z$Z"=>Dict("v$V"=>fixPR["v$V"][Pipelines["z$Z"]["To"]]^2 > fixPR["v$V"][Pipelines["z$Z"]["From"]]^2
						    ? round(Pipelines["z$Z"]["K_f"]*fixPR["v$V"][Pipelines["z$Z"]["To"]]/sqrt(fixPR["v$V"][Pipelines["z$Z"]["To"]]^2-fixPR["v$V"][Pipelines["z$Z"]["From"]]^2),digits=1)
						    : 0 for V in V_choice)
						    for Z in 1:nrow(NGNetwork_df))
	KO_minus = Dict("z$Z"=>Dict("v$V"=>fixPR["v$V"][Pipelines["z$Z"]["To"]]^2 > fixPR["v$V"][Pipelines["z$Z"]["From"]]^2
						    ? round(Pipelines["z$Z"]["K_f"]*fixPR["v$V"][Pipelines["z$Z"]["From"]]/sqrt(fixPR["v$V"][Pipelines["z$Z"]["To"]]^2-fixPR["v$V"][Pipelines["z$Z"]["From"]]^2),digits=1)
						    : 0 for V in V_choice)
						    for Z in 1:nrow(NGNetwork_df))
	# save system data
	data = Dict(:PP=>PP, :Windfarm=>Windfarm, :WindFactor=>WindFactor, :GasSupply=>GasSupply,
				:ELDemand=> ELDemand, :NGDemand=>NGDemand, :Demand=>Demand, :Lines=>Lines, :Pipelines=>Pipelines,
				:NGPr=>NGPr, :fixPR=>fixPR, :KI_plus=>KI_plus, :KO_plus=>KO_plus, :KI_minus=>KI_minus, :KO_minus=>KO_minus, :V_choice=>V_choice)
	@info("loading data done")
	return data
end

function make_indices(data,hours)
	I = ["i$n" for n=1:length(keys(data[:PP]))] #Set of dispatchable power plants
	C = ["i$n" for n=1:length(keys(data[:PP])) if data[:PP]["i$n"]["type"]=="non-NGFPP"] #Subset of dispatchable thermal power plants
	G = ["i$n" for n=1:length(keys(data[:PP])) if data[:PP]["i$n"]["type"]=="NGFPP"] #Subset of dispatchable natural gas power plants
	J = ["j$n" for n=1:length(keys(data[:Windfarm]))] #Set of windpower units
	T = ["t$n" for n=1:length(keys(data[:Demand]))-(24-hours)] #Set of time periods
	L = ["l$n" for n=1:length(keys(data[:Lines]))] #Set of electricity transmission lines
	K = ["k$n" for n=1:length(keys(data[:GasSupply]))] #Set of natural gas supply units
	Z = ["z$n" for n=1:length(keys(data[:Pipelines]))] #Set of natural gas pipelines
	MZ = ["m$n" for n=1:length(keys(data[:NGPr]))] #Set of natural gas network nodes
	ELD = ["d$n" for n=1:length(keys(data[:ELDemand]))] #Set of EL demands
	NGD = ["d$n" for n=1:length(keys(data[:NGDemand]))] #Set of NG demands
	N = ["n$n" for n=1:24] #Set of nodes
	V = ["v$n" for n=data[:V_choice]] #Set of fixed pressure points
	BigM = 100000
	indices = Dict(:I=>I,:C=>C,:G=>G,:J=>J,:T=>T,:L=>L,:K=>K,:Z=>Z,:MZ=>MZ,
				   :ELD=>ELD,:NGD=>NGD,:N=>N,:V=>V,:BigM=>BigM)
	return indices
end

function export_results(model,percentage)
	if model=="MILP"
		result_df=DataFrame(objective_value=sol_MILP[:obj],comp_time=sol_MILP[:comp_time])
		p_df = DataFrame(Tables.table(transpose(sol_MILP[:p])))
		w_df = DataFrame(Tables.table(transpose(sol_MILP[:w])))
		theta_df = DataFrame(Tables.table(transpose(sol_MILP[:theta])))
		f_df = DataFrame(Tables.table(transpose(sol_MILP[:f])))
		g_df = DataFrame(Tables.table(transpose(sol_MILP[:g])))
		pr_df = DataFrame(Tables.table(transpose(sol_MILP[:pr])))
		q_in_plus_df = DataFrame(Tables.table(transpose(sol_MILP[:q_in_plus])))
		q_in_minus_df = DataFrame(Tables.table(transpose(sol_MILP[:q_in_minus])))
		q_out_plus_df = DataFrame(Tables.table(transpose(sol_MILP[:q_out_plus])))
		q_out_minus_df = DataFrame(Tables.table(transpose(sol_MILP[:q_out_minus])))
		q_plus_df = DataFrame(Tables.table(transpose(sol_MILP[:q_plus])))
		q_minus_df = DataFrame(Tables.table(transpose(sol_MILP[:q_minus])))
		q_df = DataFrame(Tables.table(transpose(sol_MILP[:q])))
		y_df = DataFrame(Tables.table(transpose(sol_MILP[:y])))
		h_df = DataFrame(Tables.table(transpose(sol_MILP[:h])))
		phi_m_df = DataFrame(Tables.table(transpose(sol_MILP[:phi_m])))
		phi_u_df = DataFrame(Tables.table(transpose(sol_MILP[:phi_u])))
		#rename!(p_df,ind[:T])
		XLSX.writetable("MILP_result_$(percentage).xlsx",
			result=(collect(eachcol(result_df)),names(result_df)),
			p = (collect(eachcol(p_df)),names(p_df)),
			w = (collect(eachcol(w_df)),names(w_df)),
			theta = (collect(eachcol(theta_df)),names(theta_df)),
			f = (collect(eachcol(f_df)),names(f_df)),
			g = (collect(eachcol(g_df)),names(g_df)),
			pr = (collect(eachcol(pr_df)),names(pr_df)),
			q_in_plus = (collect(eachcol(q_in_plus_df)),names(q_in_plus_df)),
			q_in_minus = (collect(eachcol(q_in_minus_df)),names(q_in_minus_df)),
			q_out_plus = (collect(eachcol(q_out_plus_df)),names(q_out_plus_df)),
			q_out_minus = (collect(eachcol(q_out_minus_df)),names(q_out_minus_df)),
			q_plus = (collect(eachcol(q_plus_df)),names(q_plus_df)),
			q_minus = (collect(eachcol(q_minus_df)),names(q_minus_df)),
			q = (collect(eachcol(q_df)),names(q_df)),
			y = (collect(eachcol(y_df)),names(y_df)),
			h = (collect(eachcol(h_df)),names(h_df)),
			phi_m = (collect(eachcol(phi_m_df)),names(phi_m_df)),
			phi_u = (collect(eachcol(phi_u_df)),names(phi_u_df)))
			@info("Exported excel file for MILP at $(percentage)")
	elseif model=="LP"
		result_df=DataFrame(objective_value=sol_LP[:obj],comp_time=sol_LP[:comp_time])
		p_df = DataFrame(Tables.table(transpose(sol_LP[:p])))
		w_df = DataFrame(Tables.table(transpose(sol_LP[:w])))
		theta_df = DataFrame(Tables.table(transpose(sol_LP[:theta])))
		f_df = DataFrame(Tables.table(transpose(sol_LP[:f])))
		g_df = DataFrame(Tables.table(transpose(sol_LP[:g])))
		pr_df = DataFrame(Tables.table(transpose(sol_LP[:pr])))
		q_in_plus_df = DataFrame(Tables.table(transpose(sol_LP[:q_in_plus])))
		q_out_plus_df = DataFrame(Tables.table(transpose(sol_LP[:q_out_plus])))
		q_df = DataFrame(Tables.table(transpose(sol_LP[:q])))
		h_df = DataFrame(Tables.table(transpose(sol_LP[:h])))
		#rename!(p_df,ind[:T])
		XLSX.writetable("LP_result_$(percentage).xlsx",
			result=(collect(eachcol(result_df)),names(result_df)),
			p = (collect(eachcol(p_df)),names(p_df)),
			w = (collect(eachcol(w_df)),names(w_df)),
			theta = (collect(eachcol(theta_df)),names(theta_df)),
			f = (collect(eachcol(f_df)),names(f_df)),
			g = (collect(eachcol(g_df)),names(g_df)),
			pr = (collect(eachcol(pr_df)),names(pr_df)),
			q_in_plus = (collect(eachcol(q_in_plus_df)),names(q_in_plus_df)),
			q_out_plus = (collect(eachcol(q_out_plus_df)),names(q_out_plus_df)),
			q = (collect(eachcol(q_df)),names(q_df)),
			h = (collect(eachcol(h_df)),names(h_df)))
			@info("Exported excel file for LP at $(percentage)")
	end
end
