## Modeling Gas Flow Directions as State Variables:
# Does It Provide More Flexibility to Power Systems?
# Power Systems Computation Conference(PSCC) 2022, Electric Power Systems Research
# Code written by Junesoo Shin, s184386@student.dtu.dk
# 23.04.2022

### Set path
#cd("C:\\Users\\junes\\Desktop\\PSCC2022\\Codes\\Zenodo ver2")

### Import packages
using DataFrames
using CSV, XLSX
using Statistics
using Distributions
using Random
using JuMP
using Gurobi

### Load functions
# Data import
include("functions/func_data.jl")
# Optimization models
include("functions/func_MILP.jl")
include("functions/func_LP.jl")

### Model settings (Meshed)
mod_settings = Dict(
    # wind data volatility: "low" or "high"
    :volatility => "high",
    # model type: "MILP" or "LP"
    :model => "LP",
    # compressor: true - on, false - off
    :compressor => true,
    # linepack: true - on, false - off
    :linepack => true,
    # set gurobi relative gap -  set to default in func_MILP
    :mip_gap => 0.001,
	# total time period (max 24 hours)
	:hours => 24,
	# wind injection level from 20%~80%
	:wind_injection_level => "40%"
)

### Display output
println()
println("---------- Model settings ----------")
println("Model = $(mod_settings[:model])")
if mod_settings[:model] == "MILP"
    println("Relative gap set as default = $(mod_settings[:mip_gap])")
end
println("Wind profile = $(mod_settings[:volatility]) volatile")
println("Wind injection level = $(mod_settings[:wind_injection_level])")
println("Compressor = $(mod_settings[:compressor])")
println("Linepack = $(mod_settings[:linepack])")
println("Hours = $(mod_settings[:hours])")

### Run model
println()
println("---------- Run model ----------")
# Extract data
data = load_data(mod_settings[:volatility],mod_settings[:compressor],mod_settings[:wind_injection_level])
ind = make_indices(data,mod_settings[:hours])
if mod_settings[:model] == "MILP"
    sol_MILP = MILP(data,ind,mod_settings[:linepack],mod_settings[:mip_gap],mod_settings[:hours])
elseif mod_settings[:model] == "LP"
	sol_LP = LP(data,ind,mod_settings[:linepack],mod_settings[:hours])
end

### Model result export - saves excel sheet of all the variables
# export_results(mod_settings[:model],mod_settings[:wind_injection_level])

### Run loop for different wind injection level
# if mod_settings[:model] == "wind_injection"
# 	percentages = ["20%","30%","40%","50%","60%","70%","80%"]
# 	for i in 1:7
# 		data = load_data(mod_settings[:volatility],mod_settings[:compressor],percentages[i])
# 		ind = make_indices(data,mod_settings[:hours])
# 		@info("start solving MILP at $(percentages[i])")
# 		sol_MILP = MILP(data,ind,mod_settings[:linepack],mod_settings[:mip_gap],mod_settings[:hours])
# 		export_results("MILP",percentages[i])
# 		@info("start solving LP at $(percentages[i])")
# 		sol_LP = LP(data,ind,mod_settings[:linepack],mod_settings[:hours])
# 		export_results("LP",percentages[i])
# 	end
# end
